///Op2FF.h
//Created by Mcshay
///Last Updated: Friday, July 21st, 2006
///Generic Version

#ifndef OP2FF_H
#define OP2FF_H

#include <windows.h>
#include "Outpost2DLL.h"

#define OFFX 31
#define OFFY -1
#define MAPX 256
#define MAPY 128

///-----------------------------------Notes-------------------------------------
  /*To use the files, download them into the folder where you have your CodeBlocks
    project file. Then open the project and go to Project -> Add files... then select
    one of the Op2FF files and repeat for the other. Now both of the Op2FF files should
    be included in your project when it compiles. Finaly, you need to add the line:
        #include "Op2FF.h"
    near the top of Main.cpp (I would normaly put in below the line #include <OP2Helper.h>).

    Also, all player numbers taken by these functions need to be in the format used by Op2,
    where Player 1 is 0, Player 2 is 1, and so on. All LOCATIONs and MAP_RECTs should have
    the proper map offset already in them:
    example:
        LOCATION(5+31,10-1)
    That will make a location that correctly references the tile 5,10 on a flat map.
    If it didn't have the +31,-1, it would reference the non-existant tile -26,9.

    Make sure you change the 4 macros above this notes section for your map, OFFX should be
    +31 for flat maps, and -1 for world maps. OFFY should be -1 for all maps. MAPX and MAPY
    correspond with your maps width (MAPX) and height (MAPY).*/

///---------------Functions that handel game settings and such------------------

///SetMorale:
  /*If the game had morale steady unchecked during setup (for multiplayer) morale
    is free, otherwise it is set to the level indicated by the MoraleLevel variable.
    This variable can either be a number from 0 to 4, or one of the MoraleLevels in
    Enums.h. Any other number will cause moral to be freed by default. */
void SetMorale(int MoraleLevel);

///SetDayAndNight:
  /*If the game uses the d/n cycle, day/night shifts normaly and starts at the
    light level specified by InitLight. The normal light level is 32. If the game
    doesn't have d/n, it is always day. */
void SetDayAndNight(int InitLight);

/*Example for both SetMorale and SetDayAndNight:
    In the code:
        int InitProc()
        {
            SetMorale(moraleGood);
            SetDayAndNight(32);
            ...
        }
    The mission is started by setting moral and the d/n cycle with more commands
    later (hence the ...). */

///SetDisasters:
  //If disasters had been checked in during setup (multiplayer) disasters will occur
void SetDisasters();

///RefreshTiles:
  /*(For use with the Poof() function in IUnits.h, created by Eddy-B.) This function redraws
    the tiles around the given location, removing graphics glitches created when a unit Poof()s. */
void RefreshTiles(LOCATION loc);
/*Example:
    LOCATION Loc = x.Location();
    x.Poof();
    RefreshTiles(Loc); */

///EndGame:
  /*TextStyleKey: 0 = "MISSION ACCOMPLISHED"
                  1 = "MISSION FAILED"
                  2 = "GAME OVER"
                  3 = "MISSION ABORTED"
                  4 = "VICTORY IS SWEET"
                  5 = "YOU HAVE BEEN DEFEATED"
                  6 = "CONTROL TERMINATED"
                  7 = "OUTPOST 2 DEMO"
    Forces the specified player's game to end. The remaining params tell the game which
    sound to play and what to display as the big read text. If the TextStyle isn't one
    of the numbers shown in the TextStyleKey, it will show the contents of CustomText.
    In this way, the TextStyle would be, say -1 for a custom message. NOTE: all of the
    TextStyles above can be found within op2. Credit goes to op2Hacker for the asm.
    NOTE #2: This function ends the game in victory internaly, even if it is coded for failure.
    NOTE #3: This function will only run if the Player variable is the Local Player. */
/** |Attention|: You must follow the below instructions to use EndGame properly **/
/*DllMain Edit for EndGame:
    Start by deleting the DllMain function entirely from Main.cpp. Then replace it with this:
        extern DWORD *textPtr;
        BOOL APIENTRY DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
        {
            if (fdwReason == DLL_PROCESS_ATTACH)
                DisableThreadLibraryCalls(hinstDLL);
            else if (fdwReason == DLL_PROCESS_DETACH)
                *textPtr = 0x004E6160;

            return TRUE;
        }
    This patch to DllMain was also made by op2Hacker. */
void EndGame(int Player, int Sound, int TextStyle, char *CustomText);
/*Example:
    In the code:
        if(!x.IsLive())
            EndGame(0, sndSavnt214, -1, "NO MORE UNITS");
        //Where Unit x is (or was) a unit owned by player 1
    If Player 1's unit 'x' is dead, Player 1's game is ended with the savant voice
    "Vehical Lost", and the message "NO MORE UNITS" in red on the screen. */


///---------------------Functions that return information-----------------------

///IsAtPoint:
  /*Returns true if the first point is the same as the second point. */
bool IsAtPoint(LOCATION PointToCheck, LOCATION ReferencePoint);
/*Example:
    In the code:
        if(IsAtPoint(x.Location(), Loc))
            x.DoTransfer(1);
        //Where Unit x and LOCATION Loc have been created earlier
    Unit x is tranfered to Player 2 when it is at the location Loc */

///IsInRect:
  /*Returns true if the point is in the rect. */
bool IsInRect(LOCATION PointToCheck, MAP_RECT Rect);
/*Example:
    In the code:
        if(IsInRect(x.Location(), Rect))
            x.Destroy();
        //Where Unit x and MAP_RECT Rect have been created earlier
    Unit x is destroyed if it is in Rect. */

///GetColonyWeapon:
  /*returns mapLaser if Eden or mapMicrowave if Plymouth. Can be used to give
    all players their colony's specific starting weapons in one call to CreateUnit. */
map_id GetColonyWeapon(int PlayerNum);
/*Example:
    In the code:
        TethysGame::CreateUnit(x, mapGuardPost, Loc, 0, GetColonyWeapon(0), 0);
        //Where Unit x was defined earlier
    A GuardPost is created at Loc with Player 1's starting weapon. */

///GetColonySpecificPower:
  /*returns mapGeothermalPlant if Eden and mapMHDGenerator if Plymouth. Can be
    used to give all players their colony specific power plant in one call to CreateUnit. */
map_id GetColonySpecificPower(int PlayerNum);
/*Example:
    map_id PowerPlant = GetColonySpecificPower(0); */

///OffsetLoc:
  /*returns a LOCATION with a x value of Loc.x + Xoffset and a y value of Loc.y + Yoffset. */
LOCATION OffsetLoc(LOCATION Loc, int Xoffset, int Yoffset);
/*Example:
    In the code:
        x.DoMove(OffsetLoc(LOCATION(Loc), 5, -1));
        //Where LOCATION Loc and Unit x were defined earlier
    Unit x is told to move to the offset location Loc. */

///----------------------Functions that create things---------------------------

///ChainTubeWall:
  /*LineType Key: 0 = Straight
                  1 = Top Left Corner
                  2 = Top Right Corner
                  3 = Bottom Right Corner
                  4 = Bottom Left Corner
    Creates a line of tubes from Start to End if the Line type is Straight,
    or a line around part of the perimeter of the Rectangle created by Start
    at the top left corner and End at the bottom right corner. The Types 1-4
    create tubes along the two sides that meet at the point described. */
void ChainTubeWall(map_id mapType, int LineType, LOCATION Start, LOCATION End);
/*Example:
    ChainTubeWall(mapTube, 1, LOCATION(10+31, 5-1), LOCATION(25+31, 10-1)); */

///CreatePower:
  /*Creates a power plant of type PlantType at PlantLoc. Can be used with
    GetColonySpecificPower. */
void CreatePower(LOCATION PlantLoc, map_id PlantType, int PlayerNum, Unit &x);
/*Example:
    CreatePower(LOCATION(15+31, 10-1), mapTokamak, 0, x); */

///------------------------------Other Functions--------------------------------

///DisplayVariable:
  /*Writes the message made from the string Part1, the int Var, and the string
    Par2 (in that order) into the op2 communication window (chat). Thanks to
    op2Hacker for showing me the basic code to get this done. */
void DisplayVariable(char *Part1, int Var, char *Part2, int MaxMessageSize);
/*Example:
    The code:
        DisplayVariable("This int isn't ", 6, " it's 5.", 50);
    writes "This int isn't 6 it's 5." to the chat". */


///----------------------------------Updates------------------------------------
///Friday, July 21st, 2006:
    //Changes to EndGame made to prevent errors when not using the function
    //Added SetDisasters
///Thursday, July 20th, 2006:
    //File organised and comments typed out
    /*Added SetMorale, SetDayAndNight, RefreshTiles, EndGame, IsAtPoint, &
        IsInRect (developed for the Capture The Flag map) */
    //Updated DisplayVariable
///Lost to Time (aka: older than the 20th):
    //File idea tossed around for a month or so
    //First versions made (few functions)
    //The "add-functions-as-I-do-projects-and-not-just-for-this-file" idea surfaces
///-----------------------------------------------------------------------------

#endif
